from vinca.generators.media import media
from vinca.generators.two_lines import two_lines
from vinca.generators.verses import verses

generators_dict = {'m': media,
		   'v': verses,
		   '2': two_lines}
